@@ -1,24 +1,19 @@
 import datetime as dt
-import json
-from typing import Final, Optional, List, Literal
-
-import requests
+from typing import Optional, List
 
 from .datetime_util import total_milliseconds_since_epoch
 from .endpoints import Endpoints
 from .exceptions import IncogniaHTTPError, IncogniaError
-from .feedback_events import FeedbackEventType
 from .json_util import encode
 from .models import Coordinates, StructuredAddress, TransactionAddress, PaymentValue, PaymentMethod
 from .token_manager import TokenManager
+from .base_request import BaseRequest, JSON_CONTENT_HEADER
 
 
 class IncogniaAPI:
-    JSON_CONTENT_HEADER: Final[dict] = {'Content-type': 'application/json'}
-
-    def __init__(self, client_id: str, client_secret: str, region: Literal['br', 'us'] = 'us'):
-        self.__endpoints = Endpoints(region)
-        self.__token_manager = TokenManager(client_id, client_secret, self.__endpoints)
+    def __init__(self, client_id: str, client_secret: str):
+        self.__token_manager = TokenManager(client_id, client_secret)
+        self.__request = BaseRequest()
 
     def __get_authorization_header(self) -> dict:
         access_token, token_type = self.__token_manager.get()
@@ -34,7 +29,7 @@ class IncogniaAPI:
 
         try:
             headers = self.__get_authorization_header()
-            headers.update(self.JSON_CONTENT_HEADER)
+            headers.update(JSON_CONTENT_HEADER)
             body = {
                 'installation_id': installation_id,
                 'address_line': address_line,
@@ -42,12 +37,9 @@ class IncogniaAPI:
                 'address_coordinates': address_coordinates
             }
             data = encode(body)
-            response = requests.post(self.__endpoints.signups, headers=headers, data=data)
-            response.raise_for_status()
+            return self.__request.post(Endpoints.SIGNUPS, headers=headers, data=data)
 
-            return json.loads(response.content.decode('utf-8'))
-
-        except requests.HTTPError as e:
+        except IncogniaHTTPError as e:
             raise IncogniaHTTPError(e) from None
 
     def get_signup_assessment(self, signup_id: str) -> dict:
@@ -56,16 +48,13 @@ class IncogniaAPI:
 
         try:
             headers = self.__get_authorization_header()
-            response = requests.get(f'{self.__endpoints.signups}/{signup_id}', headers=headers)
-            response.raise_for_status()
+            return self.__request.get(f'{Endpoints.SIGNUPS}/{signup_id}', headers=headers)
 
-            return json.loads(response.content.decode('utf-8'))
-
-        except requests.HTTPError as e:
+        except IncogniaHTTPError as e:
             raise IncogniaHTTPError(e) from None
 
     def register_feedback(self,
-                          event: FeedbackEventType,
+                          event: str,
                           timestamp: dt.datetime,
                           external_id: Optional[str] = None,
                           login_id: Optional[str] = None,
@@ -80,7 +69,7 @@ class IncogniaAPI:
 
         try:
             headers = self.__get_authorization_header()
-            headers.update(self.JSON_CONTENT_HEADER)
+            headers.update(JSON_CONTENT_HEADER)
             body = {
                 'event': event,
                 'timestamp': total_milliseconds_since_epoch(timestamp),
@@ -92,10 +81,9 @@ class IncogniaAPI:
                 'installation_id': installation_id
             }
             data = encode(body)
-            response = requests.post(self.__endpoints.feedbacks, headers=headers, data=data)
-            response.raise_for_status()
+            return self.__request.post(Endpoints.FEEDBACKS, headers=headers, data=data)
 
-        except requests.HTTPError as e:
+        except IncogniaHTTPError as e:
             raise IncogniaHTTPError(e) from None
 
     def register_payment(self,
@@ -104,7 +92,8 @@ class IncogniaAPI:
                          external_id: Optional[str] = None,
                          addresses: Optional[List[TransactionAddress]] = None,
                          payment_value: Optional[PaymentValue] = None,
-                         payment_methods: Optional[List[PaymentMethod]] = None) -> dict:
+                         payment_methods: Optional[List[PaymentMethod]] = None,
+                         evaluate: Optional[bool] = None) -> dict:
         if not installation_id:
             raise IncogniaError('installation_id is required.')
         if not account_id:
@@ -112,7 +101,8 @@ class IncogniaAPI:
 
         try:
             headers = self.__get_authorization_header()
-            headers.update(self.JSON_CONTENT_HEADER)
+            headers.update(JSON_CONTENT_HEADER)
+            params = None if evaluate is None else {'eval': evaluate}
             body = {
                 'type': 'payment',
                 'installation_id': installation_id,
@@ -123,18 +113,17 @@ class IncogniaAPI:
                 'payment_methods': payment_methods
             }
             data = encode(body)
-            response = requests.post(self.__endpoints.transactions, headers=headers, data=data)
-            response.raise_for_status()
+            return self.__request.post(Endpoints.TRANSACTIONS, headers=headers, params=params,
+                                       data=data)
 
-            return json.loads(response.content.decode('utf-8'))
-
-        except requests.HTTPError as e:
+        except IncogniaHTTPError as e:
             raise IncogniaHTTPError(e) from None
 
     def register_login(self,
                        installation_id: str,
                        account_id: str,
-                       external_id: Optional[str] = None) -> dict:
+                       external_id: Optional[str] = None,
+                       evaluate: Optional[bool] = None) -> dict:
         if not installation_id:
             raise IncogniaError('installation_id is required.')
         if not account_id:
@@ -142,18 +131,17 @@ class IncogniaAPI:
 
         try:
             headers = self.__get_authorization_header()
-            headers.update(self.JSON_CONTENT_HEADER)
+            headers.update(JSON_CONTENT_HEADER)
+            params = None if evaluate is None else {'eval': evaluate}
             body = {
                 'type': 'login',
                 'installation_id': installation_id,
                 'account_id': account_id,
-                'external_id': external_id,
+                'external_id': external_id
             }
             data = encode(body)
-            response = requests.post(self.__endpoints.transactions, headers=headers, data=data)
-            response.raise_for_status()
+            return self.__request.post(Endpoints.TRANSACTIONS, headers=headers, params=params,
+                                       data=data)
 
-            return json.loads(response.content.decode('utf-8'))
-
-        except requests.HTTPError as e:
+        except IncogniaHTTPError as e:
             raise IncogniaHTTPError(e) from None
